class iaas::profile::database (
   $admin_interface		= $iaas::params::admin_interface,
   $percona_master 		= false,
   $root_password 		= undef,
   $mysql_cluster_servers	= ['hostname1', 'hostname2', 'hostname3'],
   $mysql_port 			= 3307, 
   $keystone_pwd 		= 'keystone',
   $glance_pwd 			= 'glance',
   $nova_pwd 			= 'nova',
   $cinder_pwd 			= 'cinder',
   $neutron_pwd 		= 'neutron',
   $heat_pwd 			= 'heat',
) {
	# INSTALLATION
   package{'xinetd':
    ensure	=> installed,
   } ->
   service { 'xinetd':
    ensure      => 'running'
   } ->
   class{ 'percona':
    root_password	  => $root_password,
    mysql_cluster_servers => join($mysql_cluster_servers,','),
    mysql_port 		  => $mysql_port,
    master 		  => $percona_master,
    wsrep_node_address 	  => getvar("::ipaddress_${admin_interface}") #$facts["ipaddress_${admin_interface}"],
   } ->
   exec { 'drop anonymous users on specific host':
    command => "mysql --defaults-extra-file=/root/.my.cnf -u root -e \"GRANT USAGE ON *.* TO \'\'@\'$::hostname\'; DROP USER \'\'@\'$::hostname\';\"",
    path    => [ '/bin', '/usr/bin' ],
   } ->
   exec { 'drop anonymous users on localhost':
    command => "mysql --defaults-extra-file=/root/.my.cnf -u root -e \"GRANT USAGE ON *.* TO \'\'@\'localhost\'; DROP USER \'\'@\'localhost\';\"",
    path    => [ '/bin', '/usr/bin' ],
   }   

	# DB CREATION
  $databases = ['keystone', 'glance', 'nova', 'cinder', 'neutron', 'heat' ]

  mysql_database { $databases:
     ensure  => 'present',
     charset => 'utf8',
     collate => 'utf8_general_ci',
     require => File['/root/.my.cnf'],
  }

	# DB USER
  $user_keystone = regsubst($mysql_cluster_servers, '^', 'keystone@')
  $user_glance 	 = regsubst($mysql_cluster_servers, '^', 'glance@')
  $user_nova 	 = regsubst($mysql_cluster_servers, '^', 'nova@')
  $user_cinder 	 = regsubst($mysql_cluster_servers, '^', 'cinder@')
  $user_neutron  = regsubst($mysql_cluster_servers, '^', 'neutron@')
  $user_heat 	 = regsubst($mysql_cluster_servers, '^', 'heat@')

  mysql_user { $user_keystone:
    ensure 	  => 'present',
    password_hash => mysql_password($keystone_pwd),
    require 	  => File['/root/.my.cnf']
  }
  mysql_user { $user_glance:
    ensure 	  => 'present',
    password_hash => mysql_password($glance_pwd),
    require 	  => File['/root/.my.cnf']
  }
  mysql_user { $user_nova:
    ensure 	  => 'present',
    password_hash => mysql_password($nova_pwd),
    require 	  => File['/root/.my.cnf']
  }
  mysql_user { $user_cinder:
    ensure 	  => 'present',
    password_hash => mysql_password($cinder_pwd),
    require 	  => File['/root/.my.cnf']
  }
  mysql_user { $user_neutron:
    ensure 	  => 'present',
    password_hash => mysql_password($neutron_pwd),
    require 	  => File['/root/.my.cnf']
  }
  mysql_user { $user_heat:
    ensure	  => 'present',
    password_hash => mysql_password($heat_pwd),
    require 	  => File['/root/.my.cnf']
  }

	# USER GRANT
  # defining a new resource due to old puppet and mysql versions
  # https://tickets.puppetlabs.com/browse/PUP-1263     
  define composableMySqlGrant() {
    mysql_grant {$name:
        ensure	   => 'present',
    	user 	   => regsubst($name, '\/(.)*', ''),
        table 	   => regsubst($name, '(.)*\/', ''),
        options    => ['GRANT'],
        privileges => ['ALL'],
        require    => File['/root/.my.cnf'],
      }
    }
  $grant_keystone = regsubst($user_keystone, '$', '/keystone.*')
  $grant_glance   = regsubst($user_glance, '$', '/glance.*')
  $grant_nova 	  = regsubst($user_nova, '$', '/nova.*')
  $grant_cinder   = regsubst($user_cinder, '$', '/cinder.*')
  $grant_neutron  = regsubst($user_neutron, '$', '/neutron.*')
  $grant_heat 	  = regsubst($user_heat, '$', '/heat.*')

  composableMySqlGrant{$grant_keystone: }
  composableMySqlGrant{$grant_glance: }
  composableMySqlGrant{$grant_nova: }
  composableMySqlGrant{$grant_cinder: }
  composableMySqlGrant{$grant_neutron: }
  composableMySqlGrant{$grant_heat: }

}
